@@ -7,7 +7,7 @@ use XML::Simple;
 use strict;
 use warnings;
 
-our $VERSION = '1.1.8';
+our $VERSION = '1.1.9';
 
 
 
@@ -92,12 +92,17 @@ sub createUser {
     my ( $body );
 
     $body  = $self->_xmlpre();
+
     $body .= qq(  <atom:category scheme="http://schemas.google.com/g/2005#kind" term="http://schemas.google.com/apps/2006#user" />\n);
     $body .= qq(  <apps:login userName="$arg->{'username'}" password="$arg->{'password'}" suspended="false");
+
     if ( $arg->{'passwordhashfunction'}) { 
         $arg->{'passwordhashfunction'} = uc( $arg->{'passwordhashfunction'} );
 
-        unless ( $arg->{'passwordhashfunction'} eq ( 'SHA-1' || 'MD5' ) ) {
+        unless (
+            ( $arg->{'passwordhashfunction'} eq 'SHA-1' ) ||
+            ( $arg->{'passwordhashfunction'} eq 'MD5' )
+        ) {
             croak( "Valid passwordHashFunction values are 'MD5' or 'SHA-1'" );
         }
 
@@ -106,9 +111,11 @@ sub createUser {
     else {
         $body .= qq( />\n);
     }
+
     $body .= qq(  <apps:login admin="$arg->{'admin'} />\n) if $arg->{'admin'}; 
     $body .= qq(  <apps:quota limit="$arg->{'quotalimitinmb'}" />\n) if $arg->{'quotalimitinmb'}; 
     $body .= qq(  <apps:name familyName="$arg->{'familyname'}" givenName="$arg->{'givenname'}" />\n);
+
     $body .= $self->_xmlpost();
 
     my $result = $self->_request(
@@ -220,8 +227,10 @@ sub renameUser {
     my ( $body );
 
     $body  = $self->_xmlpre();
+
     $body .= qq(  <atom:category scheme="http://schemas.google.com/g/2005#kind" term="http://schemas.google.com/apps/2006#user" />\n);
     $body .= qq(  <apps:login userName="$arg->{'newname'}" />\n);
+
     $body .= $self->_xmlpost();
 
     my $result = $self->_request(
@@ -252,6 +261,7 @@ sub updateUser {
     my ( $body );
 
     $body  = $self->_xmlpre();
+
     $body .= qq(  <atom:category scheme="http://schemas.google.com/g/2005#kind" term="http://schemas.google.com/apps/2006#user" />\n);
 
     if ( $arg->{'givenname'} || $arg->{'familyname'} ) {
@@ -265,7 +275,10 @@ sub updateUser {
         if ( $arg->{'passwordhashfunction'} ) {
             $arg->{'passwordhashfunction'} = uc( $arg->{'passwordhashfunction'} );
         
-            unless ( $arg->{'passwordhashfunction'} eq ( 'SHA-1' || 'MD5' ) ) { 
+            unless (
+                ( $arg->{'passwordhashfunction'} eq 'SHA-1' ) || 
+                ( $arg->{'passwordhashfunction'} eq 'MD5' )
+            ) { 
                 croak( "Valid passwordHashFunction values are 'MD5' or 'SHA-1'" );
             }
 
@@ -314,9 +327,20 @@ sub createGroup {
     my ( $body );
 
     $body  = $self->_xmlpre();
+
     $body .= qq(  <atom:category scheme="http://schemas.google.com/g/2005#kind" term="http://schemas.google.com/apps/2006#group" />\n);
     $body .= qq(  <apps:property name="groupId" value="$arg->{'group'}\@$self->{'domain'}" />\n);
     $body .= qq(  <apps:property name="groupName" value="$arg->{'group'}" />\n);
+
+    if ( $arg->{'description'} ) {
+        $body .= qq( <apps:property name="description" value="$arg->{'description'}"></apps:property> \n);
+    }
+
+    if ( $arg->{'permission'} ) {
+        $arg->{'permission'} = ucfirst( lc( $arg->{'permission'} ) );
+        $body .= qq( <apps:property name="emailPermission" value="$arg->{'permission'}"></apps:property> \n);
+    }
+
     $body .= $self->_xmlpost();
 
     my $result = $self->_request(
@@ -436,9 +460,11 @@ sub addGroupMember {
     my ( $body );
 
     $body  = $self->_xmlpre();
+
     $body .= qq(  <atom:category scheme="http://schemas.google.com/g/2005#kind" term="http://schemas.google.com/apps/2006#group" />\n);
     $body .= qq(  <apps:property name="groupId" value="$arg->{'group'}\@$self->{'domain'}" />\n);
     $body .= qq(  <apps:property name="memberId" value="$arg->{'member'}\@$self->{'domain'}" />\n);
+
     $body .= $self->_xmlpost();
 
     my $result = $self->_request(
@@ -538,9 +564,11 @@ sub addGroupOwner {
     my ( $body );
 
     $body  = $self->_xmlpre();
+
     $body .= qq(  <atom:category scheme="http://schemas.google.com/g/2005#kind" term="http://schemas.google.com/apps/2006#group" />\n);
     $body .= qq(  <apps:property name="groupId" value="$arg->{'group'}\@$self->{'domain'}" />\n);
     $body .= qq(  <apps:property name="email" value="$arg->{'owner'}\@$self->{'domain'}" />\n);
+
     $body .= $self->_xmlpost();
 
     my $result = $self->_request(
@@ -642,9 +670,11 @@ sub createNickname {
     my ( $body );
 
     $body  = $self->_xmlpre();
+
     $body .= qq(  <atom:category scheme="http://schemas.google.com/g/2005#kind" term="http://schemas.google.com/apps/2006#nickname" />\n);
     $body .= qq(  <apps:login userName="$arg->{'username'}" />\n);
     $body .= qq(  <apps:nickname name="$arg->{'nickname'}" />\n);
+
     $body .= $self->_xmlpost();
 
     my $result = $self->_request(
@@ -737,11 +767,27 @@ sub getUserNicknames {
 
     my ( $ref, $nickname );
 
-    foreach ( keys %{$result->{'entry'}} ) {
-        $nickname = $1 if /^.*\/(.+)$/;
+    $nickname = $result->{'entry'}->{'apps:nickname'}->{'name'};
+
+    if ( $nickname ) {
         $ref->{$nickname} = {
-            %{$result->{'entry'}->{$_}->{'apps:login'}},
-            %{$result->{'entry'}->{$_}->{'apps:nickname'}}
+            %{$result->{'entry'}->{'apps:login'}},
+            %{$result->{'entry'}->{'apps:nickname'}}
+        };
+    }
+    else {
+        foreach ( keys %{$result->{'entry'}} ) {
+            if ( /^.*\/(.+)$/ ) {
+                $nickname = $1;
+            }
+            else { next; }
+
+            next if $ref->{$nickname};
+
+            $ref->{$nickname} = {
+                %{$result->{'entry'}->{$_}->{'apps:login'}},
+                %{$result->{'entry'}->{$_}->{'apps:nickname'}}
+            };
         }
     }
 
